@@ -6,7 +6,7 @@ Convos - Multiuser IRC proxy with web interface
 
 =head1 VERSION
 
-0.81
+0.82
 
 =head1 DESCRIPTION
 
@@ -104,7 +104,8 @@ This is the URL to the Redis backend, and should follow this format:
   redis://x:password@server:port/database_index
   redis://127.0.0.1:6379/1 # suggested value
 
-Convos will use C<REDISTOGO_URL> or C<DOTCLOUD_DATA_REDIS_URL> if
+Convos will use C<REDISCLOUD_URL>, C<REDISTOGO_URL>,
+C<DOTCLOUD_DATA_REDIS_URL> or default to "redis://127.0.0.1:6379/1" unless
 C<CONVOS_REDIS_URL> is not set.
 
 It is also possible to set C<CONVOS_REDIS_INDEX=2> to use the
@@ -224,15 +225,15 @@ This template will be included below the form on the C</register> page.
 
 =over 4
 
-=item * L<Convos::Archive>
+=item * L<Convos::Controller::Archive>
 
 Mojolicious controller for IRC logs.
 
-=item * L<Convos::Client>
+=item * L<Convos::Controller::Client>
 
 Mojolicious controller for IRC chat.
 
-=item * L<Convos::User>
+=item * L<Convos::Controller::User>
 
 Mojolicious controller for user data.
 
@@ -253,7 +254,7 @@ use Convos::Core;
 use Convos::Core::Util ();
 use Convos::Upgrader;
 
-our $VERSION = '0.81';
+our $VERSION = '0.82';
 
 =head1 ATTRIBUTES
 
@@ -320,15 +321,8 @@ sub startup {
   $self->_assets;
   $self->_public_routes;
   $self->_private_routes;
+  $self->_redis_url;
 
-  if (!eval { Convos::Plugin::Helpers::REDIS_URL() } and $config->{redis}) {
-    $self->log->warn("redis url from config file will be deprecated. Run 'perldoc Convos' for alternative setup.");
-    $ENV{CONVOS_REDIS_URL} = $config->{redis};
-  }
-  if (!$ENV{CONVOS_REDIS_URL} and $self->mode eq 'production') {
-    $ENV{CONVOS_REDIS_URL} = 'redis://127.0.0.1:6379/1';
-    $self->log->info("Using default CONVOS_REDIS_URL=$ENV{CONVOS_REDIS_URL}");
-  }
   if (!$ENV{CONVOS_INVITE_CODE} and $config->{invite_code}) {
     $self->log->warn(
       "invite_code from config file will be deprecated. Set the CONVOS_INVITE_CODE env variable instead.");
@@ -355,9 +349,9 @@ sub _assets {
   my $self = shift;
 
   $self->plugin('AssetPack');
-  $self->asset('convos.css' => '/sass/convos.scss');
+  $self->asset('c.css' => '/sass/convos.scss');
   $self->asset(
-    'convos.js' => qw(
+    'c.js' => qw(
       /js/globals.js
       /js/jquery.min.js
       /js/ws-reconnecting.js
@@ -421,11 +415,10 @@ sub _from_cpan {
   my $self = shift;
   my $home = catdir dirname(__FILE__), 'Convos';
 
-  if (-d catdir $home, 'templates') {
-    $self->home->parse($home);
-    $self->static->paths->[0]   = $self->home->rel_dir('public');
-    $self->renderer->paths->[0] = $self->home->rel_dir('templates');
-  }
+  return if -d 'templates';
+  $self->home->parse($home);
+  $self->static->paths->[0]   = $self->home->rel_dir('public');
+  $self->renderer->paths->[0] = $self->home->rel_dir('templates');
 }
 
 sub _private_routes {
@@ -473,6 +466,36 @@ sub _public_routes {
   $r;
 }
 
+sub _redis_url {
+  my $self = shift;
+  my $url;
+
+  for my $k (qw( CONVOS_REDIS_URL REDISTOGO_URL REDISCLOUD_URL DOTCLOUD_DATA_REDIS_URL )) {
+    $url = $ENV{$k} or next;
+    $self->log->debug("Using $k environment variable as Redis connection URL.");
+    last;
+  }
+
+  unless ($url) {
+    if ($self->config('redis')) {
+      $self->log->warn("'redis' url from config file will be deprecated. Run 'perldoc Convos' for alternative setup.");
+      $url = $self->config('redis');
+    }
+    elsif ($self->mode eq 'production') {
+      $self->log->debug("Using default Redis connection URL redis://127.0.0.1:6379/1");
+      $url = 'redis://127.0.0.1:6379/1';
+    }
+    else {
+      $self->log->debug("Could not find CONVOS_REDIS_URL value.");
+      return;
+    }
+  }
+
+  $url = Mojo::URL->new($url);
+  $url->path($ENV{CONVOS_REDIS_INDEX}) if $ENV{CONVOS_REDIS_INDEX} and !$url->path->[0];
+  $ENV{CONVOS_REDIS_URL} = $url->to_string;
+}
+
 sub _set_secrets {
   my $self  = shift;
   my $redis = $self->redis;
