@@ -8,19 +8,11 @@ Convos::Plugin::Helpers - Mojo's little helpers
 
 use Mojo::Base 'Mojolicious::Plugin';
 use Mojo::JSON 'j';
-use Convos::Core::Util qw( format_time id_as);
-use constant DEBUG => $ENV{CONVOS_DEBUG} ? 1 : 0;
+use Convos::Core::Util qw( format_time id_as );
 use URI::Find;
-
-sub REDIS_URL {
-  $ENV{CONVOS_REDIS_URL} ||= do {
-    my $url = $ENV{REDISTOGO_URL} || $ENV{DOTCLOUD_DATA_REDIS_URL}
-      or die "CONVOS_REDIS_URL is not set. Run 'perldoc Convos' for details.\n";
-    $url = Mojo::URL->new($url);
-    $url->path($ENV{CONVOS_REDIS_INDEX}) if $ENV{CONVOS_REDIS_INDEX};
-    $url->to_string;
-  };
-}
+use constant DEBUG        => $ENV{CONVOS_DEBUG}        || 0;
+use constant DEFAULT_URL  => $ENV{DEFAULT_AVATAR_URL}  || 'https://graph.facebook.com/%s/picture?height=40&width=40';
+use constant GRAVATAR_URL => $ENV{GRAVATAR_AVATAR_URL} || 'https://gravatar.com/avatar/%s?s=40&d=retro';
 
 =head1 HELPERS
 
@@ -37,18 +29,6 @@ sub active_class {
   return ($url, $url eq $c->req->url->path ? (class => 'active') : (),);
 }
 
-=head2 avatar
-
-Used to insert an image tag.
-
-=cut
-
-sub avatar {
-  my ($self, $avatar, @args) = @_;
-
-  $self->image($self->url_for('avatar')->query(%$avatar), alt => $avatar->{user}, @args);
-}
-
 =head2 id_as
 
 See L<Convos::Core::Util/id_as>.
@@ -124,9 +104,11 @@ sub format_conversation {
 
   while (my $message = $conversation->()) {
     $message->{embed} = '';
-    $message->{uuid} ||= '';
+    $message->{uuid}   ||= '';
+    $message->{avatar} ||= '';
 
     defined $message->{message} and _parse_message($c, $message, $delay);
+    defined $message->{user} and _add_avatar($c, $message, $delay);
     push @{$c->{conversation}}, $message;
   }
 
@@ -142,6 +124,38 @@ sub format_conversation {
   $delay->begin->();    # need to do at least one step
 }
 
+sub _add_avatar {
+  my ($c, $message, $delay) = @_;
+  my $cache = $c->stash->{'convos.avatar_cache'} ||= {};
+  my $user = $message->{user};
+
+  $user =~ s!^~!!;      # somenick!~someuser@1.2.3.4
+
+  if ($cache->{$user}) {
+    return $message->{avatar} = $cache->{$user};
+  }
+
+  my $cb = $delay->begin;
+  $c->redis->hmget(
+    "user:$user",
+    qw( avatar email ),
+    sub {
+      my ($redis, $data) = @_;
+      my $id = shift @$data || shift @$data || "$user\@$message->{host}";
+
+      if ($id =~ /\@/) {
+        $message->{avatar} = sprintf GRAVATAR_URL, Mojo::Util::md5_sum($id);
+      }
+      elsif ($id) {
+        $message->{avatar} = sprintf DEFAULT_URL, $id;
+      }
+
+      $cache->{$user} = $message->{avatar};
+      $cb->();
+    }
+  );
+}
+
 sub _parse_message {
   my ($c, $message, $delay) = @_;
 
@@ -175,13 +189,13 @@ sub redis {
   my $cache_to = $c->{tx} ? 'stash' : sub { $c->app->defaults };
 
   $c->$cache_to->{redis} ||= do {
-    my $log = $c->app->log;
-    my $redis = Mojo::Redis->new(server => REDIS_URL);
+    my $log   = $c->app->log;
+    my $url   = $ENV{CONVOS_REDIS_URL} or die "CONVOS_REDIS_URL is not set. Run 'perldoc Convos' for details.\n";
+    my $redis = Mojo::Redis->new(server => $url);
 
     $redis->on(
       error => sub {
-        my ($redis, $err) = @_;
-        $log->error('[REDIS ERROR] ' . $err);
+        $log->error("[REDIS ERROR] $_[1]");
       }
     );
 
@@ -307,10 +321,9 @@ Will register the L</HELPERS> above.
 =cut
 
 sub register {
-  my ($self, $app) = @_;
+  my ($self, $app, $config) = @_;
 
   $app->helper(active_class        => \&active_class);
-  $app->helper(avatar              => \&avatar);
   $app->helper(format_conversation => \&format_conversation);
   $app->helper(connection_list     => \&connection_list);
   $app->helper(conversation_list   => \&conversation_list);
